@@ -32,7 +32,6 @@
 #include "orbitcpp/idl-compiler/error.h"
 #include "orbitcpp/idl-compiler/pass_xlate.h"
 #include <cstdlib>
-#include <strstream>
 
 #include "types/IDLTypedef.h"
 
@@ -40,8 +39,8 @@
 // IDLPassXlate --------------------------------------------------------------
 void 
 IDLPassXlate::runPass() {
-	m_header << indent << "#ifndef __ORBITCPP_IDL_" << idlUpper(m_state.m_basename) << "_COMMON" << endl
-		 << indent << "#define __ORBITCPP_IDL_" << idlUpper(m_state.m_basename) << "_COMMON" << endl
+	m_header << indent << "#ifndef ORBITCPP_IDL_" << idlUpper(m_state.m_basename) << "_COMMON" << endl
+		 << indent << "#define ORBITCPP_IDL_" << idlUpper(m_state.m_basename) << "_COMMON" << endl
 		 << endl << endl
 		 << indent << "#include <orbitcpp/orb-cpp/orbitcpp.h>" << endl
 		 << endl
@@ -100,7 +99,7 @@ IDLPassXlate::doStruct (IDL_tree  node,
 			IDLScope &scope)
 {
 	IDLStruct &strct = (IDLStruct &) *scope.getItem(node);
-
+	
 	m_header << indent << "struct " << strct.get_cpp_identifier () << endl
 		 << indent++ << "{" << endl;
 
@@ -278,8 +277,11 @@ IDLPassXlate::doUnion(IDL_tree node,IDLScope &scope)
 	m_header << indent << "class " << un.get_cpp_identifier () << endl
 		 << indent++ << "{" << endl;
 	m_header << --indent << "private:" << endl;
-	m_header << ++indent << un.get_c_typename () << " m_target;"
-		 << endl << endl;
+	m_header << ++indent << un.get_c_typename () << " " << (un.is_fixed () ? "": "*")
+		 << "m_target;" << endl << endl;
+
+	// Create constructors and destructors
+	union_create_constructor (un);
 	
 	// Create internally used methods
 	union_create_internal (un);
@@ -318,39 +320,29 @@ IDLPassXlate::doUnion(IDL_tree node,IDLScope &scope)
 	union_create_any (un);
 }
 
+
 void
-IDLPassXlate::union_create_internal (const IDLUnion &un)
+IDLPassXlate::union_create_constructor (const IDLUnion &un)
 {
-#warning "WRITE ME"
-	
-	cerr << "union_create_internal: write assignment operators" << endl;
-	
-	// _clear_member
-	m_header << indent << "void _clear_member ();" << endl;
-
-	m_module << mod_indent << "void " << un.get_cpp_method_prefix ()
-		 << "::_clear_member ()" << endl
-		 << mod_indent++ << "{" << endl;
-	m_module << mod_indent << un.get_c_method_prefix () << "__freekids ("
-		 << "&m_target, 0);" << endl;
-	m_module << --mod_indent << "}" << endl << endl;
-
 	m_header << --indent << "public: " << endl;
 	++indent;
-
-	// Wrapper constructor
-	m_header << indent << "explicit " << un.get_cpp_identifier ()
-		 << "(const " << un.get_c_typename () << " &_c_un);"
-		 << endl << endl;
-
-	m_module << mod_indent << un.get_cpp_method_prefix ()
-		 << "::" << un.get_cpp_identifier ()
-		 << " (const " << un.get_c_typename () << " &_c_un)" << endl
-		 << mod_indent++ << "{" << endl;
-	m_module << mod_indent << "_orbitcpp_unpack (_c_un);" << endl;
-	m_module << --mod_indent << "}" << endl << endl;
-
 	
+	// Create default constructor
+	m_header << indent << un.get_cpp_identifier () << "();" << endl;
+	m_module << mod_indent << un.get_cpp_method_prefix ()
+		 << "::" << un.get_cpp_identifier () << " ()";
+	if (un.is_fixed ())
+		m_module << endl;
+	else {
+		m_module << " :" << endl;
+		m_module << ++mod_indent << "m_target ("
+			 << un.get_c_typename () << "__alloc ())" << endl;
+		--mod_indent;
+	}
+
+	m_module << mod_indent << "{" << endl << mod_indent << "}" <<
+		endl << endl;
+
 	// Destructor
 	m_header << indent << "~" << un.get_cpp_identifier ()
 		 << " ();" << endl << endl;
@@ -362,6 +354,101 @@ IDLPassXlate::union_create_internal (const IDLUnion &un)
 	m_module << --mod_indent << "}" << endl << endl;
 }
 
+void
+IDLPassXlate::union_create_internal (const IDLUnion &un)
+{
+	// Wrapper constructor
+	m_header << indent << "explicit " << un.get_cpp_identifier ()
+		 << " (const " << un.get_c_typename () << " &_c_un);"
+		 << endl << endl;
+
+	m_module << mod_indent << un.get_cpp_method_prefix ()
+		 << "::" << un.get_cpp_identifier ()
+		 << " (const " << un.get_c_typename () << " &_c_un)" << endl;
+	if (!un.is_fixed ())
+	{
+		m_module << ++mod_indent << ": m_target ("
+			 << un.get_c_typename () << "__alloc ())" << endl;
+		--mod_indent;
+	}
+	m_module << mod_indent++ << "{" << endl;
+	m_module << mod_indent << "_orbitcpp_unpack (_c_un);" << endl;
+	m_module << --mod_indent << "}" << endl << endl;
+
+	// Assignment constructor
+	m_header << indent << un.get_cpp_identifier ()
+		 << " (const " << un.get_cpp_identifier () << " &un);"
+		 << endl;
+
+	m_module << mod_indent << un.get_cpp_method_prefix ()
+		 << "::" << un.get_cpp_identifier ()
+		 << " (const " << un.get_cpp_identifier () << " &un)" << endl;
+	if (!un.is_fixed ())
+	{
+		m_module << ++mod_indent << ": m_target ("
+			 << un.get_c_typename () << "__alloc ())" << endl;
+		--mod_indent;
+	}
+	m_module << mod_indent++ << "{" << endl;
+	union_create_copier (un);
+	m_module << --mod_indent << "}" << endl << endl;
+
+	// Assignment operator
+	m_header << indent << un.get_cpp_identifier ()
+		 << " & operator= (const " << un.get_cpp_identifier () << " &un);"
+		 << endl << endl;
+
+	m_module << mod_indent << un.get_cpp_typename () << " & "
+		 << un.get_cpp_method_prefix () << "::operator="
+		 << " (const " << un.get_cpp_identifier () << " &un)" << endl;
+	m_module << mod_indent++ << "{" << endl;
+	union_create_copier (un);
+	m_module << endl;
+	m_module << mod_indent << "return *this;" << endl;
+	m_module << --mod_indent << "}" << endl << endl;
+	
+	// _clear_member
+	m_header << --indent << "private:" << endl;
+	++indent;
+	m_header << indent << "void _clear_member ();" << endl << endl;
+	
+	m_module << mod_indent << "void " << un.get_cpp_method_prefix ()
+		 << "::_clear_member ()" << endl
+		 << mod_indent++ << "{" << endl;
+	if (un.is_fixed ())
+		m_module << mod_indent << "// Do nothing" << endl;
+	else
+		m_module << mod_indent << un.get_c_method_prefix () << "__freekids ("
+			 << "m_target, 0);" << endl;
+	m_module << --mod_indent << "}" << endl << endl;
+}
+
+
+void
+IDLPassXlate::union_create_copier (const IDLUnion &un)
+{
+	m_module << mod_indent << "switch (un._d ())" << endl
+		 << mod_indent << "{" << endl;
+	for (IDLUnion::const_iterator i = un.begin (); i != un.end (); ++i)
+	{
+		const IDLCaseStmt &case_stmt = static_cast<const IDLCaseStmt&> (**i);
+		const IDLMember   &member = case_stmt.get_member ();
+		const string member_name = member.get_cpp_identifier ();
+		
+		if (!case_stmt.isDefault ())
+			m_module << mod_indent++ << "case "
+				 << *(case_stmt.labelsBegin ()) << ":" << endl;
+		else
+			m_module << mod_indent++ << "default:" << endl;
+
+		m_module << mod_indent << member_name << " ("
+			 << "un." << member_name << " ());" << endl
+			 << mod_indent-- << "break;" << endl;
+	}
+	
+	m_module << mod_indent << "}" << endl;
+}
+
 
 void
 IDLPassXlate::union_create_discr (const IDLUnion &un)
@@ -377,7 +464,10 @@ IDLPassXlate::union_create_discr (const IDLUnion &un)
 	m_module << mod_indent << d_cpp << " "
 		 << un.get_cpp_method_prefix () << "::_d () const" << endl
 		 << mod_indent++ << "{" << endl;
-	m_module << "return (" << d_cpp << ")" << "m_target._d;" << endl;
+	if (un.is_fixed ())
+		m_module << "return (" << d_cpp << ")" << "m_target._d;" << endl;
+	else
+		m_module << "return (" << d_cpp << ")" << "m_target->_d;" << endl;
 	m_module << --mod_indent << "}" << endl << endl;
 
 	// Set discriminator
@@ -386,14 +476,17 @@ IDLPassXlate::union_create_discr (const IDLUnion &un)
 	m_module << mod_indent << "void " << un.get_cpp_method_prefix ()
 		 << "::_d (" << d_cpp << " d)" << endl
 		 << mod_indent++ << "{" << endl;
-	m_module << "m_target._d = (" << d_c << ") d;" << endl;
+	if (un.is_fixed ())
+		m_module << "m_target._d = (" << d_c << ") d;" << endl;
+	else
+		m_module << "m_target->_d = (" << d_c << ") d;" << endl;
 	m_module << --mod_indent << "}" << endl << endl;
 }
 
 void
 IDLPassXlate::union_create_members (const IDLUnion &un)
 {
-	for (IDLUnion::const_iterator i = un.begin (); i != un.end (); i++)
+	for (IDLUnion::const_iterator i = un.begin (); i != un.end (); ++i)
 	{
 		const IDLCaseStmt &case_stmt = static_cast<const IDLCaseStmt&> (**i);
 		const IDLMember   &member = case_stmt.get_member ();
@@ -401,7 +494,7 @@ IDLPassXlate::union_create_members (const IDLUnion &un)
 		const string member_cpp_type = member.getType ()->get_cpp_member_typename ();
 		const string member_name = member.get_cpp_identifier ();
 		
-		// Get accessor
+		// Getter accessor
 		m_header << indent << member_cpp_type << " "
 			 << member_name << " () const;" << endl;
 
@@ -412,22 +505,23 @@ IDLPassXlate::union_create_members (const IDLUnion &un)
 
 		m_module << mod_indent << member_cpp_type << " _ret;" << endl;
 
-		member.getType ()->member_unpack_from_c (m_module, mod_indent,
-							 "_ret", "m_target._u." + member_name);	
-		
+		if (un.is_fixed ())
+			member.getType ()->member_unpack_from_c (
+				m_module, mod_indent, "_ret", "m_target._u." + member_name);
+		else
+			member.getType ()->member_unpack_from_c (
+				m_module, mod_indent, "_ret", "m_target->_u." + member_name);
+			
 		m_module << mod_indent << "return _ret;" << endl;
 		m_module << --mod_indent << "}" << endl << endl;
 		
 
-		// Set accessor
-		const string discr_val = *(case_stmt.labelsBegin ());
-#if 0 // !!!
-		if(casestmt.isDefault() == true){
-			descVal = idlUnion.getDefaultDiscriminatorValue();
-		} else {
-			descVal = *(casestmt.labelsBegin());
-		}
-#endif
+		// Setter accessor
+		string discr_val;
+		if (case_stmt.isDefault ())
+			discr_val = un.get_discriminator_default_value ();
+		else
+			discr_val = *(case_stmt.labelsBegin());
 		
 		m_header << indent << "void " << member_name
 			 << " (" << member_cpp_type << " val);" << endl << endl;
@@ -439,9 +533,13 @@ IDLPassXlate::union_create_members (const IDLUnion &un)
 
 		m_module << mod_indent << "_clear_member ();" << endl;
 		m_module << mod_indent << "_d (" << discr_val << ");" << endl;
-		
-		member.getType ()->member_pack_to_c (m_module, mod_indent,
-						     "val", "m_target._u." + member_name);
+
+		if (un.is_fixed ())
+			member.getType ()->member_pack_to_c (
+				m_module, mod_indent, "val", "m_target._u." + member_name);
+		else
+			member.getType ()->member_pack_to_c (
+				m_module, mod_indent, "val", "m_target->_u." + member_name);
 
 		m_module << --mod_indent << "}" << endl << endl;
 	}
@@ -467,6 +565,7 @@ IDLPassXlate::union_create_converters (const IDLUnion &un)
 	m_module << mod_indent << "if (!_c_un)" << endl
 		 << ++mod_indent << "throw CORBA::NO_MEMORY ();" << endl << endl;
 	m_module << --mod_indent << "_orbitcpp_pack (*_c_un);" << endl;
+	m_module << mod_indent << "return _c_un;" << endl;
 	m_module << --mod_indent << "}" << endl << endl;
 	
 	// _orbitcpp_pack that works on an existing C struct
@@ -476,8 +575,12 @@ IDLPassXlate::union_create_converters (const IDLUnion &un)
 	m_module << mod_indent << "void " << method_pref
 		 << "::_orbitcpp_pack (" << c_type << " &_c_un) const" << endl
 		 << mod_indent++ << "{" << endl;
-	m_module << mod_indent << "_c_un._d = m_target._d;" << endl << endl;
-	m_module << mod_indent << "switch (_d())" << endl
+	if (un.is_fixed ())
+		m_module << mod_indent << "_c_un._d = m_target._d;" << endl << endl;
+	else
+		m_module << mod_indent << "_c_un._d = m_target->_d;" << endl << endl;
+		
+	m_module << mod_indent << "switch (_d ())" << endl
 		 << mod_indent << "{" << endl;
 
 	for (IDLUnion::const_iterator i = un.begin (); i != un.end (); ++i)
@@ -517,7 +620,6 @@ IDLPassXlate::union_create_converters (const IDLUnion &un)
 		 << "::_orbitcpp_unpack (const " << c_type << " &_c_un)" << endl
 		 << mod_indent++ << "{" << endl;
 	m_module << mod_indent << "_clear_member ();" << endl;
-	m_module << mod_indent << "m_target._d = _c_un._d;" << endl << endl;
 	
 	// Declare temporary variables
 	for (IDLUnion::const_iterator i = un.begin (); i != un.end (); ++i)
@@ -531,7 +633,9 @@ IDLPassXlate::union_create_converters (const IDLUnion &un)
 	}
 	m_module << endl;
 		
-	m_module << mod_indent << "switch (_d())" << endl
+	m_module << mod_indent << "switch ("
+		 << "(" << un.get_discriminator ().discr_get_cpp_typename () << ")"
+		 << "_c_un._d)" << endl
 		 << mod_indent << "{" << endl;
 	
 	for (IDLUnion::const_iterator i = un.begin (); i != un.end (); ++i)
