@@ -26,8 +26,8 @@
 
 
 
-#ifndef __ORBITCPP_ORB_HH
-#define __ORBITCPP_ORB_HH
+#ifndef ORBITCPP_ORB_HH
+#define ORBITCPP_ORB_HH
 
 
 
@@ -105,4 +105,4 @@ namespace CORBA {
 	};
 }
 
-#endif //__ORBITCPP_ORB_HH
+#endif //ORBITCPP_ORB_HH
