@@ -23,8 +23,8 @@
  *           Gerg rdi <cactus@cactus.rulez.org>
  */
 
-#ifndef __ORBITCPP_STRING_SEQUENCE_HH
-#define __ORBITCPP_STRING_SEQUENCE_HH
+#ifndef ORBITCPP_STRING_SEQUENCE_HH
+#define ORBITCPP_STRING_SEQUENCE_HH
 
 #include "orbitcpp/orb-cpp/orbitcpp_sequence.h"
 #include "orbitcpp/orb-cpp/orbitcpp_smartptr.h"
@@ -119,7 +119,7 @@ namespace _orbitcpp {
 	    }
 
 	// Size requisition
-	void __length (size_t new_length) {
+	void _length_set_impl (size_t new_length) {
 	    if (new_length > _max)
 	    {
 		buffer_t buffer_tmp = allocbuf (new_length);
@@ -173,7 +173,7 @@ namespace _orbitcpp {
 	    }
 	
 	// Size requisition
-	void __length (size_t new_length) {
+	void _length_set_impl (size_t new_length) {
 	    g_assert (new_length <= _max);
 	    
 	    _length = new_length;
@@ -182,5 +182,5 @@ namespace _orbitcpp {
     
 } // namespace _orbitcpp
 
-#endif // !__ORBITCPP_STRING_SEQUENCE_HH
+#endif // !ORBITCPP_STRING_SEQUENCE_HH
 
